local width, height = ...
local actors = Def.ActorFrame({})
local lists  = W_DIF:List()

local gradeToState = {
    StepMania = {
        Grade_Tier01 = 0,
        Grade_Tier02 = 1,
        Grade_Tier03 = 4,
        Grade_Tier04 = 7,
        Grade_Tier05 = 10,
        Grade_Tier06 = 13,
        Grade_Tier07 = 16,
        Grade_Failed = 19,
        Fallback = 16,
    },
    ['DDR A'] = {
        Grade_Tier01 = 0,
        Grade_Tier02 = 1,
        Grade_A3n = 2,
        Grade_A2p = 3,
        Grade_Tier03 = 4,
        Grade_A2m = 5,
        Grade_A1p = 6,
        Grade_Tier04 = 7,
        Grade_A1m = 8,
        Grade_Bp = 9,
        Grade_Tier05 = 10,
        Grade_Bm = 11,
        Grade_Cp = 12,
        Grade_Tier06 = 13,
        Grade_Cm = 14,
        Grade_Dp = 15,
        Grade_Tier07 = 16,
        Grade_Failed = 19,
        Fallback = 16,
    },
}
local gradeType = W_SCORE:GetGradeType()
-- 取得グレードActor
local function ActorGrades(difficulty)
    local actors = Def.ActorFrame({})
    for key, player in pairs(PlayerNumber) do
        -- グレード
        actors[#actors+1] = Def.ActorFrame({
            InitCommand = function(self)
                self:x(((player == PLAYER_1) and -1 or 1) * (width/2+20))
            end,
            LoadActor('../../add_grade')..{
                InitCommand = function(self)
                    self:animate(false)
                    self:visible(false)
                    self:xy(-6,1)
                end,
                CallbackStepsMessageCommand = function(self, params)
                    if not params.Song then
                        self:visible(false)
                        return
                    end
                    gradeType = W_SCORE:GetGradeType()
                    if params.Player == player and params.Steps then
                        self:visible(false)
                        if params.Grade[params.Player][difficulty] then
                            self:visible(true)
                            self:setstate(gradeToState[gradeType][params.Grade[params.Player][difficulty]] or gradeToState[gradeType].Fallback)
                        end
                    end
                end,
            },
            -- フルコンボ
            LoadActor('../../fullcombo')..{
                InitCommand = function(self)
                    self:animate(false)
                    self:visible(false)
                    self:xy(12,2)
                end,
                CallbackStepsMessageCommand = function(self, params)
                    if not params.Song then
                        self:visible(false)
                        return
                    end
                    if params.Player == player and params.Steps then
                        self:visible(false)
                        if params.FullCombo[params.Player][difficulty] then
                            self:visible(true)
                            if params.FullCombo[params.Player][difficulty] == 1 then
                                self:setstate(1)
                                self:diffuse(GameColor.Judgment["JudgmentLine_W1"])
                                self:glowblink()
                                self:effectperiod(0.20)
                                self:zoom(1)
                            elseif params.FullCombo[params.Player][difficulty] == 2 then
                                self:setstate(1)
                                self:diffuse(GameColor.Judgment["JudgmentLine_W2"])
                                self:glowshift()
                                self:zoom(0.85)
                            elseif params.FullCombo[params.Player][difficulty] == 3 then
                                self:setstate(1)
                                self:diffuse(GameColor.Judgment["JudgmentLine_W3"])
                                self:stopeffect()
                                self:zoom(0.7)
                            elseif params.FullCombo[params.Player][difficulty] == 4 then
                                self:setstate(2)
                                self:diffuse(BoostColor(GameColor.Judgment["JudgmentLine_W4"],1.5))
                                self:stopeffect()
                                self:zoom(1)
                            end
                        end
                    end
                end,
            }
        })
    end
    return actors
end

for difficulty,name in pairs(lists) do
    actors[#actors+1] = Def.ActorFrame({
        InitCommand = function(self)
            self:y(Enum.Reverse(Difficulty)[difficulty]*height)
        end,
        -- 難易度名
        LoadFont('Common Bold')..{
            Text = name,
            InitCommand = function(self)
                self:zoom(0.55)
                self:maxwidth((width-40)/0.55)
                self:horizalign(left)
                self:x(-width / 2 + 5)
                self:diffuse(W_DIF:Color(difficulty))
                --self:strokecolor(Color('Black'))
            end,
            CallbackStepsMessageCommand = function(self, params)
                self:diffusealpha(0.5)
                local song = params.Song
                if not song then
                    return
                end
                local steps = song:GetOneSteps(params.StepsType, difficulty)
                if steps then
                    self:strokecolor(steps:IsAutogen() and {1,1,1,0.3} or Color('Black'))
                    self:diffusealpha(1)
                end
            end,
        },
        -- 難易度
        LoadFont('Common Bold')..{
            InitCommand = function(self)
                self:zoom(0.75)
                self:maxwidth(30/0.75)
                self:horizalign(right)
                self:x(width / 2 - 5)
                self:diffuse(W_DIF:Color(difficulty))
                self:strokecolor(Color('Black'))
            end,
            CallbackStepsMessageCommand = function(self, params)
                self:diffusealpha(0.5)
                local song = params.Song
                if not song then
                    self:settext('-')
                    return
                end
                local steps = song:GetOneSteps(params.StepsType, difficulty)
                if steps then
                    self:strokecolor(steps:IsAutogen() and {1,1,1,0.3} or Color('Black'))
                    -- OFでは同じ難易度で複数譜面を作成できるので数をチェックする
                    local all = song:GetAllSteps()
                    local difCounts = {}
                    for i = 1, #all do
                        if all[i]:GetStepsType() == params.StepsType then
                            local difName = (all[i]:IsAnEdit() or all[i]:IsAPlayerEdit())
                                and 'Difficulty_Edit'
                                or all[i]:GetDifficulty()
                            if not difCounts[difName] then
                                difCounts[difName] = 1
                            else
                                difCounts[difName] = difCounts[difName]+1
                            end
                        end
                    end
                    -- 複数ある場合は難易度の数字ではなく譜面数を表示
                    if difCounts[difficulty] and difCounts[difficulty] > 1 then
                        self:settext('x'..difCounts[difficulty])
                    else
                        self:settext(steps:GetMeter())
                    end
                    self:diffusealpha(1)
                else
                    self:settext('-')
                end
            end,
        },
        ActorGrades(difficulty),
    })
end

return actors