local n = 'WAIEI CitMin'
local v = '3.0.16'
local d = '20250313'
local textVer = 'HotFix-1'

local function DebugCode(self)
    return ''
end

local function Key()
    return string.sub(THEME:GetCurThemeName(), 1, 12)
end

local cnvVer = {
    Fix   = 10000,
    RC    = 2000,
    Beta  = 1000,
    Alpha = 0,
}
local function Version(self, ...)
    local mode = ...
    -- 直接使うと上書きしてしまうので代入
    local local_v, local_textVer = v, textVer
    if mode then
        if type(mode) == 'table' then
            -- 指定したバージョンの数値を返却
            local_v       = mode.Main or mode[1] or v
            local_textVer = mode.Sub or mode[2] or (mode[1] and '' or textVer)
        elseif mode == 'main' then
            return local_v
        elseif mode == 'sub' then
            return local_textVer
        end
    end
    local hf = 0
    if string.sub(local_textVer, 1, 6) == 'HotFix' then
        hf = tonumber(split('-', local_textVer)[2])
        local_textVer = ''
    end
    local tmp = split('%.', local_v)
    local numVer = tonumber(tmp[1])*10000000    -- シリーズバージョン
                + tonumber(tmp[2])*100000       -- マイナーバージョン（MAX100）
    local tmp1, tmp2 = string.match(local_textVer, '^([^-]+)-(%d+).*$')
    tmp = {tmp1, tmp2, tmp[3]}
    -- HotFix（MAX9）
    numVer = numVer + (tmp[1] and cnvVer[tmp[1]] or cnvVer.Fix) + tonumber(tmp[2] or 0) + tonumber(tmp[3] or 0)*10 + hf
    return numVer
end

-- 数値化したバージョンからテキスト表記を取得
local function ConvertVersion(self, numVer)
    local main = string.format('%d.%d.%d',
                        math.floor(numVer/10000000),
                        (numVer > 30010016) and math.floor((math.floor(numVer/100000)%100)/10) or math.floor(numVer/100000)%100,
                        (math.floor((numVer%100000)/10000) == 1) and numVer%1000 or 0)
    local hf = (numVer > 30010016 and numVer%10 >= 1) and 'HotFix-'..(numVer%10) or ''
    local subVer = numVer%100000
    local sub = ''
    for _,sv in pairs({'Fix', 'RC', 'Beta', 'Alpha'}) do
        if subVer >= cnvVer[sv] then
            sub = string.format('%s-%d', sv, subVer%1000)
            break
        end
    end
    -- 正式版ならsubバージョンを表示しない
    if string.match(sub, '^Fix%-%d+') then
        return main
    end
    return main..' '..sub
end

local function GetDateVersion(self)
    return tonumber(d)
end

-- 5.0系かどうか
local function Is50(self)
    return (YA_VER:Version() < 5100)
end

-- 5.1系かどうか
local function Is51(self)
    return (YA_VER:Version() >= 5100 and YA_VER:Version() < 5190)
end

-- Project OutFoxかどうか
local function IsOF(self)
    return (YA_VER:Version() >= 5300)
end

-- Project OutFox 4系かどうか
local function IsOF4(self)
    return (YA_VER:Version() >= 5300 and YA_VER:Version() < 530500)
end

-- Project OutFoxのAlphaV以降かどうか
local function IsOFV(self)
    return (YA_VER:Version() >= 530500)
end

-- 汎用画像を取得(Graphics/Objects)
local function GetPathObject(self, value)
    return THEME:GetPathG('', 'Objects/'..value)
end

-- 現在のpnのstepsを取得
local function Steps(self, pn)
    if GAMESTATE:IsCourseMode() then
        local trail = GAMESTATE:GetCurrentTrail(pn)
        if trail then
            local st = math.max(GAMESTATE:GetCourseSongIndex() or 0, 0)
            local entry = trail:GetTrailEntry(st)
            return entry:GetSteps()
        end
    end
    return GAMESTATE:GetCurrentSteps(pn)
end

-- 現在のsongを取得
local function Song(self)
    if GAMESTATE:IsCourseMode() then
        local trail = GAMESTATE:GetCurrentTrail(GAMESTATE:GetMasterPlayerNumber())
        if trail then
            local st = math.max(GAMESTATE:GetCourseSongIndex() or 0, 0)
            local entry = trail:GetTrailEntry(st)
            local song = entry and entry:GetSong()
            return song or SONGMAN:GetRandomSong()
        end
    end
    return GAMESTATE:GetCurrentSong()
end

-- プレイヤー1かどうか
local function IsPlayer1(self, pn)
    return pn == PLAYER_1
end

-- 相手側のプレイヤー
local function AnotherPlayer(self, pn)
    return W:IsP1(pn) and PLAYER_2 or PLAYER_1
end

-- プレイヤーから数値に変換
-- (pn == PLAYER_1) and 1 or 2
local function GetPlayerEnum(self, pn)
    return Enum.Reverse(PlayerNumber)[pn] + 1
end

-- SEとして扱う
local function SetSE(self, _self)
    if not W:Is50() then _self:set_is_action(true) end
end

local scriptDir = THEME:GetCurrentThemeDirectory()..'Scripts/'
-- 画面切り替え
W_BRANCHE = dofile(scriptDir..'dofile/Branches.lua')
-- 難易度
W_DIF  = dofile(scriptDir..'dofile/Difficulty.lua')
-- 判定
W_JUDGMENT = dofile(scriptDir..'dofile/Judgment.lua')
-- 画像
W_GRAPHICS = dofile(scriptDir..'dofile/Graphics.lua')
-- ステージ
W_STAGE = dofile(scriptDir..'dofile/Stage.lua')
-- プレイヤー
W_PLAYER = dofile(scriptDir..'dofile/Player.lua')
-- スコア
W_SCORE = dofile(scriptDir..'dofile/Score.lua')
-- 文字列
W_STRING = dofile(scriptDir..'dofile/String.lua')
-- タイマー
W_TIMER = dofile(scriptDir..'dofile/Timer.lua')
-- アバター
W_AVATAR = dofile(scriptDir..'dofile/Avatar.lua')
-- ターゲットスコア
W_TARGET = dofile(scriptDir..'dofile/Target.lua')
-- ダンスステージ・キャラクター
W_DANCER = dofile(scriptDir..'dofile/Dancer.lua')
-- フォルダ
W_FOLDER = dofile(scriptDir..'dofile/Folder.lua')
-- カスタムプラグイン
W_CUSTOM = dofile(scriptDir..'dofile/Custom.lua')
-- オプション
W_OPTION = dofile(scriptDir..'dofile/Option.lua')
-- ライフ
W_LIFE = dofile(scriptDir..'dofile/Life.lua')
-- オンライン
W_NET = dofile(scriptDir..'dofile/Net.lua')

-- グループスキャン処理時にDefaultMeterTypeの設定を適用する
local function Scan(self)
    local defaultMeterType = W_OPTION:ReadPref('DefaultMeterType', 'DDR')
    YA_LIB.GROUP:AddKey('MeterType', 'string', {
        Default = '',
        DDR     = 'DDR',
        DDRX    = 'DDR X',
        X       = 'DDR X',
        ITG     = 'ITG',
    })
    YA_GROUP:AfterFunction('MeterType', function(value)
        return (value == '') and defaultMeterType or value
    end)
    YA_GROUP:Scan()
end

local reload
local function ReloadScreen(self, screenName, ...)
    local params = ...
    reload = {Name = screenName, Params = params or {}}
    SCREENMAN:SetNewScreen('ScreenReloadScreen')
end

local function GetReloadScreenName(self)
    return reload and reload.Name or 'ScreenInit'
end

local function GetReloadParameters(self, ...)
    local key = ...
    local params = (reload or {Params = {}}).Params or {}
    -- 何度も呼ばれる可能性があるので原則値のクリアは行わない
    if key then
        return params[key] or nil
    end
    return params
end

local function DateNumber(self)
    return Year()*10000000000+(MonthOfYear()+1)*100000000+(DayOfMonth()+1)*1000000+Hour()*10000+Minute()*100+Second()
end

-- Encrypt
local function LuaEncrypt(self, path)
    local i = 0
    local md5 = CRYPTMAN:MD5String(Key())
    local md5_size = string.len(md5)
    local f_org = RageFileUtil.CreateRageFile()
    local f_dst = RageFileUtil.CreateRageFile()
    if f_org:Open(path..'_org.lua', 1) then
        local org = f_org:Read()
        local dst = ''
        if f_dst:Open(path..'_enc.lua', 2) then
            local size = string.len(org)
            while i < size do
                i = i+1
                local byte = ((string.byte(org, i)+(string.byte(md5, ((i-1)%md5_size)+1)))%255)
                byte = (byte == 0) and 255 or byte
                dst = dst..string.char(byte)
            end
            f_dst:Write(dst)
            f_dst:Close()
        end
        f_dst:destroy()
        f_org:Close()
    end
    f_org:destroy()
end

-- Decrypt
local function LuaDecrypt(self, path, ...)
    local output = ...
    if output == nil then output = false end
    if not FILEMAN:DoesFileExist(path) then return nil end
    local i = 0
    local md5 = CRYPTMAN:MD5String(Key())
    local md5_size = string.len(md5)
    local f_org = RageFileUtil.CreateRageFile()
    local dst = 'return {Actor=Def.ActorFrame({}),Config={"Off"}}'
    if f_org:Open(path, 1) then
        local org = f_org:Read()
        if string.sub(org, 1, 5) ~= '-- NE' then    -- Not Encrypt
            local size = string.len(org)
            dst = ''
            while i < size do
                i = i+1
                local byte = ((string.byte(org, i)-(string.byte(md5, ((i-1)%md5_size)+1))+255)%255)
                byte = (byte == 255) and 0 or byte
                dst = dst..string.char(byte)
            end
        else
            dst = org
        end
        f_org:Close()
    end
    f_org:destroy()
    local luaString, errMsg
    -- loadstringはLua5.2以降廃止
    -- assertでエラーをキャッチするとluaDataの中身が出力されるのでここではキャッチしない
    if _VERSION == 'Lua 5.1' then
        luaString, errMsg = loadstring(dst)
    else
        luaString, errMsg = load(dst)
    end
    dst = nil
    return luaString
end

-- OF 0.5用、ディレクトリ内のファイル情報再取得
local function FileFlush(self, path)
    if FILEMAN.FlushDirCache then
        local dirs = split('/', path)
        dirs[#dirs] = nil
        FILEMAN:FlushDirCache(join('/', dirs)..'/')
    end
end

W = {
    Debug   = DebugCode,
    Name    = function(self) return n end,
    Ver     = Version,
    StrVer  = function(self) return v..((textVer ~= '') and ' '..textVer or '') end,
    DateVer = GetDateVersion,
    CnvVer  = ConvertVersion,
    DateNumber = DateNumber,            -- 日付・時刻を連結した数値にする
    Is50    = Is50,                     -- 5.0系かどうか
    Is51    = Is51,                     -- 5.1系かどうか
    IsOF    = IsOF,                     -- OutFoxかどうか
    IsOF4   = IsOF4,                    -- OutFox 0.4系かどうか
    IsOFV   = IsOFV,                    -- OutFox AlphaVかどうか
    Object  = GetPathObject,            -- 汎用画像を取得
    Steps   = Steps,                    -- 現在のステップ
    Song    = Song,                     -- 現在の楽曲
    IsP1    = IsPlayer1,                -- プレイヤー1かどうか
    APn     = AnotherPlayer,            -- 相手側のプレイヤー
    PNum    = GetPlayerEnum,            -- プレイヤーから数値に変換
    SE      = SetSE,                    -- SEとして扱う
    Reload  = ReloadScreen,             -- 画面を再読み込み
    ReloadName = GetReloadScreenName,   -- 再読み込み対象のスクリーン名
    ReloadParams = GetReloadParameters, -- 再読み込み前に渡したパラメータを取得

    LuaE = LuaEncrypt,
    LuaD = LuaDecrypt,
    Flush = FileFlush,

    GroupScan = Scan,            -- WAIEI Coreのスキャンを上書き
}

--[[
WAIEI CitMin.lua

Copyright (c) 2021 A.C

This software is released under the MIT License.
https://opensource.org/licenses/mit-license.php
--]]
